﻿//---------------------------------------------------------------------------
//
//   ViewPoint
//
//          Copyright(C) 2014, Tark
//
//  頂点座標を表示
//
//---------------------------------------------------------------------------

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <memory>
#include <sstream>

#include "ViewPointWindow.h"
#include "ViewPoint.h"
#include "MQSetting.h"

static ViewPoint plugin;

// Constructor
// コンストラクタ
ViewPoint::ViewPoint() : m_Window(nullptr), m_bActivate(false), m_bUpdate(true)
{
	setlocale(LC_CTYPE, "");
}

// Destructor
// デストラクタ
ViewPoint::~ViewPoint()
{
	if(m_Window != NULL){
		delete m_Window;
		m_Window = NULL;
	}
}

//---------------------------------------------------------------------------
//  GetPlugInID
//---------------------------------------------------------------------------
void ViewPoint::GetPlugInID(DWORD *Product, DWORD *ID)
{
	*Product = 0xE3F7A830;
	*ID = 0xBD9A1880;
}

//---------------------------------------------------------------------------
//  GetPlugInName
//---------------------------------------------------------------------------
const char *ViewPoint::GetPlugInName(void)
{
	return "ViewPoint Ver.1.1   Copyright(C) 2014, Tark.";
}

//---------------------------------------------------------------------------
//  EnumString
//---------------------------------------------------------------------------
const char *ViewPoint::EnumString(void)
{
	return "ViewPoint";
}

//---------------------------------------------------------------------------
//  Initialize
//---------------------------------------------------------------------------
BOOL ViewPoint::Initialize()
{
/*	int posX = -1, posY = -1;
	{
		MQSetting *setting = OpenSetting();
		setting->Load("ViewPoint_activate", m_bActivate, false);
		setting->Load("update", m_bUpdate, false);
		setting->Load("PosX", posX, -1);
		setting->Load("PosY", posY, -1);
		CloseSetting(setting);
	}
*/
	if (m_Window == NULL){
		MQWindowBase mainwnd = MQWindow::GetMainWindow();
		m_Window = new ViewPointWindow(this, mainwnd);
/*		if(posX >= 0 && posY >= 0) {
			m_Window->SetPosX(posX);
			m_Window->SetPosY(posY);
		}
*/	}
	Activate(nullptr, m_bActivate);
	return TRUE;
}

//---------------------------------------------------------------------------
//  Exit
//    アプリケーションの終了
//---------------------------------------------------------------------------
void ViewPoint::Exit()
{
/*	{
		MQSetting *setting = OpenSetting();
		setting->Save("ViewPoint_activate", m_bActivate);
		setting->Save("update", m_bUpdate);
		setting->Save("all", m_bAll);
		setting->Save("visible", m_bVisible);
		setting->Save("freeze", m_bFreeze);
		setting->Save("triangle", m_bTriangle);
		setting->Save("polygon", m_bPolygon);
		setting->Save("PosX", m_Window->GetPosX());
		setting->Save("PosY", m_Window->GetPosY());
		CloseSetting(setting);
	}
*/	if (m_Window != NULL){
		delete m_Window;
		m_Window = NULL;
	}
}

//---------------------------------------------------------------------------
//  Activate
//---------------------------------------------------------------------------
BOOL ViewPoint::Activate(MQDocument doc, BOOL flag)
{
//	m_doc = doc;
	if (m_Window == NULL) {
		return FALSE;
	}
	m_bActivate = flag ? true : false;
	m_Window->SetVisible(m_bActivate);

	return m_bActivate;
}

//---------------------------------------------------------------------------
//  IsActivated
//---------------------------------------------------------------------------
BOOL ViewPoint::IsActivated(MQDocument doc)
{
	if (m_Window == NULL) {
		return FALSE;
	}
	clear();
	return m_bActivate;
}

//---------------------------------------------------------------------------
//  ExecuteCallback
///---------------------------------------------------------------------------
bool ViewPoint::ExecuteCallback(MQDocument doc, void *option)
{
	CallbackInfo *info = (CallbackInfo*)option;
	return ((*this).*info->proc)(doc);
}

// コールバックの呼び出し
void ViewPoint::Execute(ExecuteCallbackProc proc)
{
	CallbackInfo info;
	info.proc = proc;
	BeginCallback(&info);
}

// プラグインのベースクラスを返す
MQBasePlugin *GetPluginClass()
{
	return &plugin;
}

void ViewPoint::OnNewDocument(MQDocument doc_, const char *filename, NEW_DOCUMENT_PARAM& param)
{
	clear();
}

void ViewPoint::OnDraw(MQDocument doc_, MQScene scene, int width, int height)
{
	viewPoint(doc_);
}

void ViewPoint::OnObjectSelected(MQDocument doc_)
{
	viewPoint(doc_);
}

void ViewPoint::OnObjectModified(MQDocument doc_)
{
	viewPoint(doc_);
}

void ViewPoint::OnUpdateObjectList(MQDocument doc_)
{
	clear();
}

BOOL ViewPoint::OnUndo(MQDocument doc_, int undo_state)
{
	viewPoint(doc_);
	return FALSE;
}

BOOL ViewPoint::OnRedo(MQDocument doc_, int redo_state)
{
	viewPoint(doc_);
	return FALSE;
}

BOOL ViewPoint::clear()
{
	if(!m_bActivate || !m_bUpdate) {
		return FALSE;
	}
	m_Window->clear();
	return FALSE;
}

BOOL ViewPoint::viewPoint(MQDocument doc_) {
	if(!m_bActivate || !m_bUpdate) {
		return FALSE;
	}
	int count = 0;
	MQPoint p(0, 0, 0);
	cmq::CMQDocument doc(doc_);
	cmq::CMQObject cur(doc.getCurrentObject());
	cmq::CMQContainer<cmq::CMQVertex, cmq::CMQObject> vcontainer(&cur);
	for(auto vit = vcontainer.begin(); vit != vcontainer.end(); ++vit) {
		if(vit->isSelect()) {
			if(count == 0) {
				p = cur.getVertex(vit.index());
				count++;
			}
			else {
				break;
			}
		}
	}
	if(count == 1) {
		m_Window->viewPoint(p);
		return TRUE;
	}
	else {
		m_Window->clear();
		return FALSE;
	}
}
